// Process.cpp
// The main computation goes here

#include <windows.h>
#include <commctrl.h>
#include <stdio.h>
#include "plugin.h"

// Simple brightness and contrast filter, input 0-100, 50=no change
void ImageIt(int preview, int brite, int contr, MINIGRAPHIC *src,
	     MINIGRAPHIC *dst)
{
  unsigned char britemap[256]; 
  double sx, ex, dx, val;
  unsigned int i, x, y;
  unsigned char *line, *out;
  unsigned char min, max, mid;
  int delta;

  // Input checking
  if (brite<0) brite = 0;
  if (brite>100) brite = 100;
  if (contr<0) contr = 0;
  if (contr>100) contr = 100;

  if (brite==50) {
    // 50 => no change
    for (i=0; i<256; i++) britemap[i] = i;
  } else if (brite<50) {
    // Map all intensities down from 255-E...0
    sx = 0.0;
    ex = 255 - (50-brite)*200/50;
    dx = (ex-sx)/256;
    for (i=0; i<256; i++) britemap[i] = (unsigned char)(sx+dx*i);
  } else {
    // Map all intensitied up from E...255
    sx = (brite-50)*200/50;
    ex = 255;
    dx = (ex-sx)/256;
    for (i=0; i<256; i++) britemap[i] = (unsigned char)(sx+dx*i);
  }
	
  // Now table-lookup the entire image
  for (y=0; y<src->height; y++) {
    line = src->bits+src->lineWidth*y;
    out = dst->bits+dst->lineWidth*y;
    for (x=0; x<src->width*3; x++) {
      *(out++) = britemap[*(line++)];
    }
  }

  if (contr==50) return; // No contrast change, we're done!

  // Find min and max in image
  max = 0;
  min = 255;
  for (y=0; y<dst->height; y++) {
    out = dst->bits+dst->lineWidth*y;
    for (x=0; x<dst->width*3; x++) {
      if (*out > max) max = *out;
      if (*out < min) min = *out;
      out++;
    }
  }

  // Use those limits to figure out the slope of the contrast function
  mid = (min+max)/2;
  if (contr<50)
    dx = 1.0 - (50.0-contr)/50.0;
  else
    dx = 1.0 + (contr-50.0)/20.0;
  if (dx<0.01) dx=0.01;

  // Use the DX to generate a linear mapping
  for (i=0; i<256; i++) {
    delta = i-mid;
    val=(mid+delta*dx);
    if (val<0) val = 0;
    if (val>255) val = 255;
    britemap[i] = (unsigned char)val;
  }

  // Again, table-lookup
  for (y=0; y<dst->height; y++) {
    line = dst->bits+dst->lineWidth*y;
    for (x=0; x<dst->width*3; x++) {
      *(line) = britemap[*(line)];
      line++;
    }
  }

  // All done!
}

